import { ExifToolTask } from "./ExifToolTask";
import { WriteTags } from "./WriteTags";
export declare function htmlEncode(s: string): string;
export declare const DefaultWriteTaskOptions: {
    readonly useMWG: boolean;
};
export type WriteTaskOptions = typeof DefaultWriteTaskOptions;
export interface WriteTaskResult {
    /**
     * Number of files created by ExifTool
     */
    created: number;
    /**
     * Number of files updated by ExifTool. Note that this does not mean any
     * field values were _changed_ from prior values.
     */
    updated: number;
    /**
     * Number of files that ExifTool knew it did not need change. Note that
     * ExifTool (at least as of v12.70) only realizes it doesn't need to change
     * a file if you are clearing an already empty value.
     */
    unchanged: number;
    /**
     * Non-exceptional warnings from ExifTool, like "Error: Nothing to write",
     * or "Nothing to do."
     *
     * Any invalid tag names or values will cause {@link Error}s to be thrown.
     */
    warnings?: string[];
}
export declare class WriteTask extends ExifToolTask<WriteTaskResult> {
    readonly sourceFile: string;
    readonly args: string[];
    private constructor();
    static for(filename: string, tags: WriteTags, extraArgs?: string[], options?: Partial<WriteTaskOptions>): WriteTask;
    toString(): string;
    protected parse(data: string, error?: Error): WriteTaskResult;
}
